#version 150 core

uniform int mode;

uniform int hasShadow;

uniform vec3 light_ambient;
uniform vec3 light_diffuse;
uniform vec3 light_specular;

uniform vec3 vertColor;

uniform vec3 mat_ambient;
uniform vec3 mat_diffuse;
uniform vec3 mat_specular;
uniform float mat_shininess;

uniform int useTexture;
uniform sampler2D shadowMap;
uniform sampler2D texture;



smooth in vec3 N;
smooth in vec3 E;
smooth in vec3 L;
smooth in vec2 UV;
smooth in vec4 smCoord;

out vec4 outColor;


void shadowPass(void){}

void finalPass(void){
	if (useTexture == 0){
		outColor = vec4(vertColor,1.0f);
	}else{
		outColor = texture2D(texture, UV);
	}
	
	

	vec4 ambient = vec4(light_ambient,outColor.a);
	vec4 diffuse = vec4(light_diffuse,outColor.a);
	vec4 specular= vec4(light_specular,outColor.a);

	//float NdotL = max(0.0, dot(normalize(N), L));
	float NdotL = dot(normalize(N), L);
	float EdotL = dot(normalize(E),L);

	diffuse *= NdotL;

	

	if(NdotL < 0.0f) outColor = outColor*0.9;
	else outColor += diffuse;

	if(hasShadow == 1.0f && smCoord.w > 0.0f){
		float depth = texture2DProj(shadowMap, smCoord).z;
		vec3 shadowCoord = smCoord.xyz / smCoord.w;
		if(depth < shadowCoord.z) outColor *= 0.8f;
		
		
	}
}

void main(void)
{
	if(mode == 0) shadowPass();
	else if(mode == 1) finalPass();
}
