#version 150 core

uniform int mode;

uniform mat4 boneMatrices[60];

uniform mat4 smMatrix;
uniform mat4 lpvMatrix;

uniform mat4 pMatrix;
uniform mat4 vMatrix;
uniform mat4 mMatrix;
uniform mat3 normalMatrix;

uniform vec3 cameraPos;
uniform vec3 lightPos;

uniform vec3 light_ambient;
uniform vec3 light_diffuse;
uniform vec3 light_specular;

in vec4 vertPos;
in vec3 vertNormal;
in vec2 vertUv;
in vec3 vertColor;
in vec4 boneWeight;
in vec4 boneIndex;

smooth out vec2 UV;
smooth out vec3 N;
smooth out vec3 E;
smooth out vec3 L;
smooth out vec4 smCoord;

void shadowPass(void){
	vec4 outBoneWeight = boneWeight;
	outBoneWeight.w = 1.0 - dot(boneWeight.xyz, vec3(1.0f, 1.0f, 1.0f));

	mat4 transformMatrix = outBoneWeight.x * boneMatrices[int(boneIndex.x)];
	transformMatrix += outBoneWeight.y * boneMatrices[int(boneIndex.y)];
	transformMatrix += outBoneWeight.z * boneMatrices[int(boneIndex.z)];
	transformMatrix += outBoneWeight.w * boneMatrices[int(boneIndex.w)];
	
	vec4 newPosition = transformMatrix * vertPos;
	vec4 newNormal = transformMatrix * vec4(vertNormal, 0.0f);
	
	mat4 mvpMatrix = pMatrix * vMatrix * mMatrix;

	gl_Position = mvpMatrix * newPosition;

}

void finalPass(void){
	vec4 outBoneWeight = boneWeight;
	outBoneWeight.w = 1.0 - dot(boneWeight.xyz, vec3(1.0f, 1.0f, 1.0f));

	mat4 transformMatrix = outBoneWeight.x * boneMatrices[int(boneIndex.x)];
	transformMatrix += outBoneWeight.y * boneMatrices[int(boneIndex.y)];
	transformMatrix += outBoneWeight.z * boneMatrices[int(boneIndex.z)];
	transformMatrix += outBoneWeight.w * boneMatrices[int(boneIndex.w)];
	
	vec4 newPosition = transformMatrix * vertPos;
	vec4 newNormal = transformMatrix * vec4(vertNormal, 0.0f);
	

	mat4 mvpMatrix = pMatrix * vMatrix * mMatrix;


	vec4 worldVertexPos = mMatrix * newPosition;

	vec3 worldVertexPosN = vec3(normalize(worldVertexPos));
	
	vec4 worldLightPos = vec4(lightPos, 1.0);
	vec3 worldLightPosN = vec3(normalize(worldLightPos));
	
	vec4 worldCameraPos = vec4(cameraPos, 1.0);
	vec3 worldCameraPosN = vec3(normalize(worldCameraPos));

	N = normalize(normalMatrix * vec3(newNormal));
	//Eyedirection
	E = normalize(cameraPos);
	//Lightdirection
	L = normalize(worldLightPosN - worldVertexPosN);
	
	UV = vertUv;
	
	//Lightviewspace-lookup
	smCoord = smMatrix * mMatrix * newPosition;

	//Cameraviewspace
	gl_Position = mvpMatrix * newPosition;
}

void main(void)
{
	if(mode == 0) shadowPass();
	else if(mode == 1) finalPass();
}